@@ -1,19 +1,21 @@
 //! Data types that track the change state for syntax nodes.
 
-use rustc_hash::FxHashMap;
-
-use crate::parse::syntax::{Syntax, SyntaxId};
+use crate::{
+    hash::DftHashMap,
+    parse::syntax::{Syntax, SyntaxId},
+};
 
 #[derive(PartialEq, Eq, Clone, Copy)]
 pub enum ChangeKind<'a> {
     Unchanged(&'a Syntax<'a>),
     ReplacedComment(&'a Syntax<'a>, &'a Syntax<'a>),
+    ReplacedString(&'a Syntax<'a>, &'a Syntax<'a>),
     Novel,
 }
 
 #[derive(Debug, Default)]
 pub struct ChangeMap<'a> {
-    changes: FxHashMap<SyntaxId, ChangeKind<'a>>,
+    changes: DftHashMap<SyntaxId, ChangeKind<'a>>,
 }
 
 impl<'a> ChangeMap<'a> {
