@@ -1,7 +1,7 @@
 //! A graph representation for computing tree diffs.
 
 use bumpalo::Bump;
-use rustc_hash::FxHashMap;
+use hashbrown::hash_map::RawEntryMut;
 use std::{
     cell::{Cell, RefCell},
     cmp::min,
@@ -15,6 +15,7 @@ use crate::{
         changes::{insert_deep_unchanged, ChangeKind, ChangeMap},
         stack::Stack,
     },
+    hash::DftHashMap,
     parse::syntax::{AtomKind, Syntax, SyntaxId},
 };
 use Edge::*;
@@ -276,6 +277,7 @@ impl<'s, 'b> Vertex<'s, 'b> {
 pub enum Edge {
     UnchangedNode {
         depth_difference: u32,
+        probably_punctuation: bool,
     },
     EnterUnchangedDelimiter {
         depth_difference: u32,
@@ -283,70 +285,60 @@ pub enum Edge {
     ReplacedComment {
         levenshtein_pct: u8,
     },
-    NovelAtomLHS {
-        contiguous: bool,
-        probably_punctuation: bool,
-    },
-    NovelAtomRHS {
-        contiguous: bool,
-        probably_punctuation: bool,
+    ReplacedString {
+        levenshtein_pct: u8,
     },
+    NovelAtomLHS {},
+    NovelAtomRHS {},
     // TODO: An EnterNovelDelimiterBoth edge might help performance
     // rather doing LHS and RHS separately.
-    EnterNovelDelimiterLHS {
-        contiguous: bool,
-    },
-    EnterNovelDelimiterRHS {
-        contiguous: bool,
-    },
+    EnterNovelDelimiterLHS {},
+    EnterNovelDelimiterRHS {},
 }
 
-const NOT_CONTIGUOUS_PENALTY: u32 = 50;
-
 impl Edge {
     pub fn cost(self) -> u32 {
         match self {
             // Matching nodes is always best.
-            UnchangedNode { depth_difference } => min(40, depth_difference + 1),
+            UnchangedNode {
+                depth_difference,
+                probably_punctuation,
+            } => {
+                // TODO: Perhaps prefer matching longer strings? It's
+                // probably easier to read.
+
+                // The cost for unchanged nodes can be as low as 1,
+                // but we penalise nodes that have a different depth
+                // difference, capped at 40.
+                let base = min(40, depth_difference + 1);
+
+                // If the node is only punctuation, increase the
+                // cost. It's better to have unchanged variable names
+                // and novel punctuation than the reverse.
+                //
+                // We want a sufficiently large punctuation cost such
+                // that unchanged variables always win, even if there
+                // are replacement edges elsewhere.
+                //
+                // Replacement edges have a cost between 500 and 600,
+                // so they can be up to 100 less than two novel nodes.
+                // If we have replacements either side of a node
+                // (e.g. see comma_and_comment_before.js), then that's
+                // potentially a cost difference of 200.
+                base + if probably_punctuation { 200 } else { 0 }
+            }
             // Matching an outer delimiter is good.
             EnterUnchangedDelimiter { depth_difference } => 100 + min(40, depth_difference),
 
-            // Replacing a comment is better than treating it as novel.
-            ReplacedComment { levenshtein_pct } => 150 + u32::from(100 - levenshtein_pct),
-
             // Otherwise, we've added/removed a node.
-            NovelAtomLHS {
-                contiguous,
-                probably_punctuation,
-            }
-            | NovelAtomRHS {
-                contiguous,
-                probably_punctuation,
-            } => {
-                let mut cost = 300;
-                if !contiguous {
-                    cost += NOT_CONTIGUOUS_PENALTY;
-                }
-                // If it's only punctuation, decrease the cost
-                // slightly. It's better to have novel punctuation
-                // than novel variable names.
-                if probably_punctuation {
-                    cost -= 10;
-                }
-                cost
-            }
-            EnterNovelDelimiterLHS { contiguous } | EnterNovelDelimiterRHS { contiguous } => {
-                let mut cost = 300;
-                if !contiguous {
-                    // This needs to be more than 40 greater than the
-                    // contiguous case. Otherwise, we end up choosing
-                    // a badly positioned unchanged delimiter just
-                    // because it has a better depth difference.
-                    //
-                    // TODO: write a test for this case.
-                    cost += NOT_CONTIGUOUS_PENALTY;
-                }
-                cost
+            NovelAtomLHS {} | NovelAtomRHS {} => 300,
+            EnterNovelDelimiterLHS { .. } | EnterNovelDelimiterRHS { .. } => 300,
+            // Replacing a comment is better than treating it as
+            // novel. However, since ReplacedComment is an alternative
+            // to NovelAtomLHS and NovelAtomRHS, we need to be
+            // slightly less than 2 * 300.
+            ReplacedComment { levenshtein_pct } | ReplacedString { levenshtein_pct } => {
+                500 + u32::from(100 - levenshtein_pct)
             }
         }
     }
@@ -355,10 +347,14 @@ impl Edge {
 fn allocate_if_new<'s, 'b>(
     v: Vertex<'s, 'b>,
     alloc: &'b Bump,
-    seen: &mut FxHashMap<&Vertex<'s, 'b>, Vec<&'b Vertex<'s, 'b>>>,
+    seen: &mut DftHashMap<&Vertex<'s, 'b>, Vec<&'b Vertex<'s, 'b>>>,
 ) -> &'b Vertex<'s, 'b> {
-    match seen.get_mut(&v) {
-        Some(existing) => {
+    // We use the entry API so that we only need to do a single lookup
+    // for access and insert.
+    match seen.raw_entry_mut().from_key(&v) {
+        RawEntryMut::Occupied(mut occupied) => {
+            let existing = occupied.get_mut();
+
             // Don't explore more than two possible parenthesis
             // nestings for each syntax node pair.
             if let Some(allocated) = existing.last() {
@@ -381,7 +377,7 @@ fn allocate_if_new<'s, 'b>(
             existing.push(allocated);
             allocated
         }
-        None => {
+        RawEntryMut::Vacant(vacant) => {
             let allocated = alloc.alloc(v);
 
             // We know that this vec will never have more than 2
@@ -392,18 +388,21 @@ fn allocate_if_new<'s, 'b>(
             let mut existing: Vec<&'b Vertex<'s, 'b>> = Vec::with_capacity(2);
             existing.push(allocated);
 
-            seen.insert(allocated, existing);
+            vacant.insert(allocated, existing);
             allocated
         }
     }
 }
 
-/// Does this atom look like punctuation?
+/// Does this node look like punctuation?
 ///
 /// This check is deliberately conservative, becuase it's hard to
 /// accurately recognise punctuation in a language-agnostic way.
-fn looks_like_punctuation(content: &str) -> bool {
-    content == "," || content == ";" || content == "."
+fn looks_like_punctuation(node: &Syntax) -> bool {
+    match node {
+        Syntax::Atom { content, .. } => content == "," || content == ";" || content == ".",
+        _ => false,
+    }
 }
 
 /// Pop as many parents of `lhs_node` and `rhs_node` as
@@ -474,18 +473,18 @@ fn pop_all_parents<'s>(
 }
 
 /// Compute the neighbours of `v` if we haven't previously done so,
-/// write them to the .neighbours cell inside `v`, and return them.
-pub fn get_set_neighbours<'s, 'b>(
+/// and write them to the .neighbours cell inside `v`.
+pub fn set_neighbours<'s, 'b>(
     v: &Vertex<'s, 'b>,
     alloc: &'b Bump,
-    seen: &mut FxHashMap<&Vertex<'s, 'b>, Vec<&'b Vertex<'s, 'b>>>,
-) -> Vec<(Edge, &'b Vertex<'s, 'b>)> {
-    match &*v.neighbours.borrow() {
-        Some(neighbours) => return neighbours.clone(),
-        None => {}
+    seen: &mut DftHashMap<&Vertex<'s, 'b>, Vec<&'b Vertex<'s, 'b>>>,
+) {
+    if v.neighbours.borrow().is_some() {
+        return;
     }
 
-    let mut res: Vec<(Edge, &Vertex)> = vec![];
+    // There are only seven pushes in this functions, so that's sufficient.
+    let mut res: Vec<(Edge, &Vertex)> = Vec::with_capacity(7);
 
     if let (Some(lhs_syntax), Some(rhs_syntax)) = (&v.lhs_syntax, &v.rhs_syntax) {
         if lhs_syntax == rhs_syntax {
@@ -493,6 +492,8 @@ pub fn get_set_neighbours<'s, 'b>(
                 - rhs_syntax.num_ancestors() as i32)
                 .unsigned_abs();
 
+            let probably_punctuation = looks_like_punctuation(lhs_syntax);
+
             // Both nodes are equal, the happy case.
             let (lhs_syntax, rhs_syntax, lhs_parent_id, rhs_parent_id, parents) = pop_all_parents(
                 lhs_syntax.next_sibling(),
@@ -503,7 +504,10 @@ pub fn get_set_neighbours<'s, 'b>(
             );
 
             res.push((
-                UnchangedNode { depth_difference },
+                UnchangedNode {
+                    depth_difference,
+                    probably_punctuation,
+                },
                 allocate_if_new(
                     Vertex {
                         neighbours: RefCell::new(None),
@@ -580,21 +584,26 @@ pub fn get_set_neighbours<'s, 'b>(
         if let (
             Syntax::Atom {
                 content: lhs_content,
-                kind: AtomKind::Comment,
+                kind: lhs_kind @ AtomKind::Comment | lhs_kind @ AtomKind::String,
                 ..
             },
             Syntax::Atom {
                 content: rhs_content,
-                kind: AtomKind::Comment,
+                kind: rhs_kind @ AtomKind::Comment | rhs_kind @ AtomKind::String,
                 ..
             },
         ) = (lhs_syntax, rhs_syntax)
         {
-            // Both sides are comments and their content is reasonably
-            // similar.
-            if lhs_content != rhs_content {
+            // Both sides are comments/both sides are strings and
+            // their content is reasonably similar.
+            if lhs_kind == rhs_kind && lhs_content != rhs_content {
                 let levenshtein_pct =
                     (normalized_levenshtein(lhs_content, rhs_content) * 100.0).round() as u8;
+                let edge = if lhs_kind == &AtomKind::Comment {
+                    ReplacedComment { levenshtein_pct }
+                } else {
+                    ReplacedString { levenshtein_pct }
+                };
 
                 let (lhs_syntax, rhs_syntax, lhs_parent_id, rhs_parent_id, parents) =
                     pop_all_parents(
@@ -605,7 +614,7 @@ pub fn get_set_neighbours<'s, 'b>(
                         &v.parents,
                     );
                 res.push((
-                    ReplacedComment { levenshtein_pct },
+                    edge,
                     allocate_if_new(
                         Vertex {
                             neighbours: RefCell::new(None),
@@ -627,11 +636,7 @@ pub fn get_set_neighbours<'s, 'b>(
     if let Some(lhs_syntax) = &v.lhs_syntax {
         match lhs_syntax {
             // Step over this novel atom.
-            Syntax::Atom { content, .. } => {
-                // TODO: should this apply if prev is a parent node
-                // rather than a sibling?
-                let contiguous = lhs_syntax.prev_is_contiguous();
-
+            Syntax::Atom { .. } => {
                 let (lhs_syntax, rhs_syntax, lhs_parent_id, rhs_parent_id, parents) =
                     pop_all_parents(
                         lhs_syntax.next_sibling(),
@@ -642,10 +647,7 @@ pub fn get_set_neighbours<'s, 'b>(
                     );
 
                 res.push((
-                    NovelAtomLHS {
-                        contiguous,
-                        probably_punctuation: looks_like_punctuation(content),
-                    },
+                    NovelAtomLHS {},
                     allocate_if_new(
                         Vertex {
                             neighbours: RefCell::new(None),
@@ -667,8 +669,6 @@ pub fn get_set_neighbours<'s, 'b>(
 
                 let parents_next = push_lhs_delimiter(&v.parents, lhs_syntax);
 
-                let contiguous = lhs_syntax.prev_is_contiguous();
-
                 let (lhs_syntax, rhs_syntax, lhs_parent_id, rhs_parent_id, parents) =
                     pop_all_parents(
                         lhs_next,
@@ -679,7 +679,7 @@ pub fn get_set_neighbours<'s, 'b>(
                     );
 
                 res.push((
-                    EnterNovelDelimiterLHS { contiguous },
+                    EnterNovelDelimiterLHS {},
                     allocate_if_new(
                         Vertex {
                             neighbours: RefCell::new(None),
@@ -701,9 +701,7 @@ pub fn get_set_neighbours<'s, 'b>(
     if let Some(rhs_syntax) = &v.rhs_syntax {
         match rhs_syntax {
             // Step over this novel atom.
-            Syntax::Atom { content, .. } => {
-                let contiguous = rhs_syntax.prev_is_contiguous();
-
+            Syntax::Atom { .. } => {
                 let (lhs_syntax, rhs_syntax, lhs_parent_id, rhs_parent_id, parents) =
                     pop_all_parents(
                         v.lhs_syntax,
@@ -714,10 +712,7 @@ pub fn get_set_neighbours<'s, 'b>(
                     );
 
                 res.push((
-                    NovelAtomRHS {
-                        contiguous,
-                        probably_punctuation: looks_like_punctuation(content),
-                    },
+                    NovelAtomRHS {},
                     allocate_if_new(
                         Vertex {
                             neighbours: RefCell::new(None),
@@ -737,7 +732,6 @@ pub fn get_set_neighbours<'s, 'b>(
             Syntax::List { children, .. } => {
                 let rhs_next = children.get(0).copied();
                 let parents_next = push_rhs_delimiter(&v.parents, rhs_syntax);
-                let contiguous = rhs_syntax.prev_is_contiguous();
 
                 let (lhs_syntax, rhs_syntax, lhs_parent_id, rhs_parent_id, parents) =
                     pop_all_parents(
@@ -749,7 +743,7 @@ pub fn get_set_neighbours<'s, 'b>(
                     );
 
                 res.push((
-                    EnterNovelDelimiterRHS { contiguous },
+                    EnterNovelDelimiterRHS {},
                     allocate_if_new(
                         Vertex {
                             neighbours: RefCell::new(None),
@@ -772,8 +766,7 @@ pub fn get_set_neighbours<'s, 'b>(
         "Must always find some next steps if node is not the end"
     );
 
-    v.neighbours.replace(Some(res.clone()));
-    res
+    v.neighbours.replace(Some(res));
 }
 
 pub fn populate_change_map<'s, 'b>(
@@ -798,13 +791,20 @@ pub fn populate_change_map<'s, 'b>(
                 change_map.insert(lhs, ChangeKind::Unchanged(rhs));
                 change_map.insert(rhs, ChangeKind::Unchanged(lhs));
             }
-            ReplacedComment { levenshtein_pct } => {
+            ReplacedComment { levenshtein_pct } | ReplacedString { levenshtein_pct } => {
                 let lhs = v.lhs_syntax.unwrap();
                 let rhs = v.rhs_syntax.unwrap();
+                let change_kind = |first, second| {
+                    if let ReplacedComment { .. } = e {
+                        ChangeKind::ReplacedComment(first, second)
+                    } else {
+                        ChangeKind::ReplacedString(first, second)
+                    }
+                };
 
-                if *levenshtein_pct > 40 {
-                    change_map.insert(lhs, ChangeKind::ReplacedComment(lhs, rhs));
-                    change_map.insert(rhs, ChangeKind::ReplacedComment(rhs, lhs));
+                if *levenshtein_pct > 20 {
+                    change_map.insert(lhs, change_kind(lhs, rhs));
+                    change_map.insert(rhs, change_kind(rhs, lhs));
                 } else {
                     change_map.insert(lhs, ChangeKind::Novel);
                     change_map.insert(rhs, ChangeKind::Novel);
