@@ -5,7 +5,7 @@ use crate::{
     display::context::{calculate_after_context, calculate_before_context, opposite_positions},
     display::hunks::Hunk,
     display::style::{self, apply_colors, apply_line_number_color},
-    lines::{format_line_num, split_on_newlines, MaxLine},
+    lines::{format_line_num, MaxLine},
     options::DisplayOptions,
     parse::syntax::MatchedPos,
     summary::FileFormat,
@@ -18,8 +18,8 @@ pub fn print(
     lhs_positions: &[MatchedPos],
     rhs_positions: &[MatchedPos],
     hunks: &[Hunk],
-    lhs_display_path: &str,
-    rhs_display_path: &str,
+    display_path: &str,
+    old_path: &Option<String>,
     file_format: &FileFormat,
 ) {
     let (lhs_colored_lines, rhs_colored_lines) = if display_options.use_color {
@@ -43,14 +43,8 @@ pub fn print(
         )
     } else {
         (
-            split_on_newlines(lhs_src)
-                .iter()
-                .map(|s| format!("{}\n", s))
-                .collect(),
-            split_on_newlines(rhs_src)
-                .iter()
-                .map(|s| format!("{}\n", s))
-                .collect(),
+            lhs_src.lines().map(|s| format!("{}\n", s)).collect(),
+            rhs_src.lines().map(|s| format!("{}\n", s)).collect(),
         )
     };
 
@@ -70,8 +64,8 @@ pub fn print(
         println!(
             "{}",
             style::header(
-                lhs_display_path,
-                rhs_display_path,
+                display_path,
+                old_path,
                 i + 1,
                 hunks.len(),
                 file_format,
