@@ -28,6 +28,7 @@ mod diff;
 mod display;
 mod exit_codes;
 mod files;
+mod hash;
 mod line_parser;
 mod lines;
 mod options;
@@ -94,7 +95,8 @@ fn reset_sigpipe() {
 
 /// The entrypoint.
 fn main() {
-    pretty_env_logger::init_timed();
+    pretty_env_logger::try_init_timed_custom_env("DFT_LOG")
+        .expect("The logger has not been previously initalized");
     reset_sigpipe();
 
     match options::parse_args() {
