@@ -7,7 +7,9 @@ use const_format::formatcp;
 use crossterm::tty::IsTty;
 
 use crate::{
-    display::style::BackgroundColor, exit_codes::EXIT_BAD_ARGUMENTS, parse::guess_language,
+    display::style::BackgroundColor,
+    exit_codes::EXIT_BAD_ARGUMENTS,
+    parse::guess_language::{language_override_from_name, LanguageOverride},
 };
 
 pub const DEFAULT_BYTE_LIMIT: usize = 1_000_000;
@@ -208,11 +210,27 @@ inline: A single column display, closer to traditional diff display.")
                 .help("Treat paths that don't exist as equivalent to an empty file. Only applies when diffing files, not directories.")
         )
         .arg(
-            Arg::new("language").long("language")
-                .value_name("EXT")
-                .allow_invalid_utf8(true)
-                .help("Override language detection. Inputs are assumed to have this file extension. When diffing directories, applies to all files.")
-                // TODO: support DFT_LANGUAGE for consistency
+            Arg::new("override").long("override")
+                .value_name("GLOB:NAME")
+                .help(concat!("Associate this glob pattern with this language, overriding normal language detection. For example:
+
+$ ", env!("CARGO_BIN_NAME"), " --override='*.c:C++' old.c new.c
+
+Language names are matched case insensitively. Overrides may also specify the language \"text\" to treat a file as plain text.
+
+This argument may be given more than once. For example:
+
+$ ", env!("CARGO_BIN_NAME"), " --override='CustomFile:json' --override='*.c:text' old.c new.c
+
+To configure multiple overrides using environment variables, difftastic also accepts DFT_OVERRIDE_1 up to DFT_OVERRIDE_9.
+
+$ export DFT_OVERRIDE='CustomFile:json'
+$ export DFT_OVERRIDE_1='*.c:text'
+$ export DFT_OVERRIDE_2='*.js:javascript jsx'
+
+When multiple overrides are specified, the first matching override wins."))
+                .env("DFT_OVERRIDE")
+                .multiple_occurrences(true)
         )
         .arg(
             Arg::new("list-languages").long("list-languages")
@@ -327,53 +345,118 @@ pub enum Mode {
         diff_options: DiffOptions,
         display_options: DisplayOptions,
         set_exit_code: bool,
-        language_override: Option<guess_language::Language>,
+        language_overrides: Vec<(glob::Pattern, LanguageOverride)>,
         /// The path where we can read the LHS file. This is often a
         /// temporary file generated by source control.
         lhs_path: FileArgument,
         /// The path where we can read the RHS file. This is often a
         /// temporary file generated by source control.
         rhs_path: FileArgument,
-        /// The path that we should display for the LHS file. This is
-        /// usually the same as `lhs_path`, but the six argument form
-        /// of git-diff specifies this separately.
-        lhs_display_path: String,
-        /// The path that we should display for the RHS file.
-        rhs_display_path: String,
+        /// The path that we show to the user.
+        display_path: String,
+        /// If this file has been renamed, the name it had previously.
+        old_path: Option<String>,
     },
     ListLanguages {
         use_color: bool,
+        language_overrides: Vec<(glob::Pattern, LanguageOverride)>,
     },
     DumpTreeSitter {
         path: String,
-        language_override: Option<guess_language::Language>,
+        language_overrides: Vec<(glob::Pattern, LanguageOverride)>,
     },
     DumpSyntax {
         path: String,
-        language_override: Option<guess_language::Language>,
         ignore_comments: bool,
+        language_overrides: Vec<(glob::Pattern, LanguageOverride)>,
     },
 }
 
+fn common_path_suffix(lhs_path: &Path, rhs_path: &Path) -> Option<String> {
+    let lhs_rev_components = lhs_path
+        .components()
+        .map(|c| c.as_os_str())
+        .rev()
+        .collect::<Vec<_>>();
+    let rhs_rev_components = rhs_path
+        .components()
+        .map(|c| c.as_os_str())
+        .rev()
+        .collect::<Vec<_>>();
+
+    let mut common_components = vec![];
+    for (lhs_component, rhs_component) in lhs_rev_components.iter().zip(rhs_rev_components.iter()) {
+        if lhs_component == rhs_component {
+            common_components.push(lhs_component.to_string_lossy());
+        } else {
+            break;
+        }
+    }
+
+    if common_components.is_empty() {
+        None
+    } else {
+        common_components.reverse();
+        Some(common_components.join(&std::path::MAIN_SEPARATOR.to_string()))
+    }
+}
+
+fn build_display_path(lhs_path: &FileArgument, rhs_path: &FileArgument) -> String {
+    match (lhs_path, rhs_path) {
+        (FileArgument::NamedPath(lhs), FileArgument::NamedPath(rhs)) => {
+            match common_path_suffix(lhs, rhs) {
+                Some(common_suffix) => common_suffix,
+                None => rhs.display().to_string(),
+            }
+        }
+        (FileArgument::NamedPath(p), _) | (_, FileArgument::NamedPath(p)) => {
+            p.display().to_string()
+        }
+        (FileArgument::DevNull, _) | (_, FileArgument::DevNull) => "/dev/null".into(),
+        (FileArgument::Stdin, FileArgument::Stdin) => "-".into(),
+    }
+}
+
+fn parse_overrides_or_die(raw_overrides: &[String]) -> Vec<(glob::Pattern, LanguageOverride)> {
+    let mut res = vec![];
+    let mut invalid_syntax = false;
+
+    for raw_override in raw_overrides {
+        if let Some((glob_str, lang_name)) = raw_override.rsplit_once(':') {
+            match glob::Pattern::new(glob_str) {
+                Ok(pattern) => {
+                    if let Some(language_override) = language_override_from_name(lang_name) {
+                        res.push((pattern, language_override));
+                    } else {
+                        eprintln!("No such language '{}'", lang_name);
+                        eprintln!("See --list-languages for the names of all languages available. Language overrides are case insensitive.");
+                        invalid_syntax = true;
+                    }
+                }
+                Err(e) => {
+                    eprintln!("Invalid glob syntax '{}'", glob_str);
+                    eprintln!("Glob parsing error: {}", e.msg);
+                    invalid_syntax = true;
+                }
+            }
+        } else {
+            eprintln!("Invalid language override syntax '{}'", raw_override);
+            eprintln!("Language overrides are in the format 'GLOB:LANG_NAME', e.g. '*.js:JSON'.");
+            invalid_syntax = true;
+        }
+    }
+
+    if invalid_syntax {
+        std::process::exit(EXIT_BAD_ARGUMENTS);
+    }
+
+    res
+}
+
 /// Parse CLI arguments passed to the binary.
 pub fn parse_args() -> Mode {
     let matches = app().get_matches();
 
-    let language_override = match matches.value_of_os("language") {
-        Some(lang_str) => {
-            if let Some(lang) = guess_language::from_extension(lang_str) {
-                Some(lang)
-            } else {
-                eprintln!(
-                    "No language is associated with extension: {}",
-                    lang_str.to_string_lossy()
-                );
-                None
-            }
-        }
-        None => None,
-    };
-
     let color_output = match matches.value_of("color").expect("color has a default") {
         "always" => ColorOutput::Always,
         "never" => ColorOutput::Never,
@@ -386,22 +469,37 @@ pub fn parse_args() -> Mode {
 
     let ignore_comments = matches.is_present("ignore-comments");
 
+    let mut raw_overrides: Vec<String> = vec![];
+    if let Some(overrides) = matches.values_of("override") {
+        raw_overrides = overrides.map(|s| s.into()).collect();
+    }
+    for i in 1..=9 {
+        if let Ok(value) = env::var(format!("DFT_OVERRIDE_{}", i)) {
+            raw_overrides.push(value);
+        }
+    }
+
+    let language_overrides = parse_overrides_or_die(&raw_overrides);
+
     if matches.is_present("list-languages") {
-        return Mode::ListLanguages { use_color };
+        return Mode::ListLanguages {
+            use_color,
+            language_overrides,
+        };
     }
 
     if let Some(path) = matches.value_of("dump-syntax") {
         return Mode::DumpSyntax {
             path: path.to_string(),
-            language_override,
             ignore_comments,
+            language_overrides,
         };
     }
 
     if let Some(path) = matches.value_of("dump-ts") {
         return Mode::DumpTreeSitter {
             path: path.to_string(),
-            language_override,
+            language_overrides,
         };
     }
 
@@ -409,25 +507,20 @@ pub fn parse_args() -> Mode {
     info!("CLI arguments: {:?}", args);
 
     // TODO: document these different ways of calling difftastic.
-    let (lhs_display_path, rhs_display_path, lhs_path, rhs_path, in_vcs) = match &args[..] {
+    let (display_path, lhs_path, rhs_path, old_path, in_vcs) = match &args[..] {
         [lhs_path, rhs_path] => {
             let lhs_arg = FileArgument::from_cli_argument(lhs_path);
             let rhs_arg = FileArgument::from_cli_argument(rhs_path);
-            (
-                lhs_arg.display(),
-                rhs_arg.display(),
-                lhs_arg,
-                rhs_arg,
-                false,
-            )
+            let display_path = build_display_path(&lhs_arg, &rhs_arg);
+            (display_path, lhs_arg, rhs_arg, None, false)
         }
         [display_path, lhs_tmp_file, _lhs_hash, _lhs_mode, rhs_tmp_file, _rhs_hash, _rhs_mode] => {
             // https://git-scm.com/docs/git#Documentation/git.txt-codeGITEXTERNALDIFFcode
             (
-                display_path.to_string_lossy().to_string(),
                 display_path.to_string_lossy().to_string(),
                 FileArgument::from_path_argument(lhs_tmp_file),
                 FileArgument::from_path_argument(rhs_tmp_file),
+                None,
                 true,
             )
         }
@@ -435,11 +528,15 @@ pub fn parse_args() -> Mode {
         {
             // Rename file.
             // TODO: where does git document these 9 arguments?
+
+            let old_name = old_name.to_string_lossy().to_string();
+            let new_name = new_name.to_string_lossy().to_string();
+
             (
-                old_name.to_string_lossy().to_string(),
-                new_name.to_string_lossy().to_string(),
+                new_name,
                 FileArgument::from_path_argument(lhs_tmp_file),
                 FileArgument::from_path_argument(rhs_tmp_file),
+                Some(old_name),
                 true,
             )
         }
@@ -545,11 +642,11 @@ pub fn parse_args() -> Mode {
         diff_options,
         display_options,
         set_exit_code,
-        language_override,
+        language_overrides,
         lhs_path,
         rhs_path,
-        lhs_display_path,
-        rhs_display_path,
+        display_path,
+        old_path,
     }
 }
 
