@@ -1,8 +1,8 @@
 //! Load and configure parsers written with tree-sitter.
 
-use std::collections::HashMap;
 use std::collections::HashSet;
 
+use crate::hash::DftHashMap;
 use crate::options::DiffOptions;
 use crate::parse::guess_language as guess;
 use tree_sitter as ts;
@@ -425,7 +425,9 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             let language = unsafe { tree_sitter_haskell() };
             TreeSitterConfig {
                 language,
-                atom_nodes: vec![].into_iter().collect(),
+                atom_nodes: vec!["qualified_variable", "qualified_module"]
+                    .into_iter()
+                    .collect(),
                 delimiter_tokens: vec![("[", "]"), ("(", ")")],
                 highlight_query: ts::Query::new(
                     language,
@@ -857,7 +859,10 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             let language = unsafe { tree_sitter_scala() };
             TreeSitterConfig {
                 language,
-                atom_nodes: vec!["string", "template_string"].into_iter().collect(),
+                // TODO: probably all comments should be treated as atoms
+                atom_nodes: vec!["string", "template_string", "comment", "block_comment"]
+                    .into_iter()
+                    .collect(),
                 delimiter_tokens: vec![("{", "}"), ("(", ")"), ("[", "]")],
                 highlight_query: ts::Query::new(
                     language,
@@ -1035,8 +1040,8 @@ pub fn parse_subtrees(
     src: &str,
     config: &TreeSitterConfig,
     tree: &tree_sitter::Tree,
-) -> HashMap<usize, (tree_sitter::Tree, TreeSitterConfig, HighlightedNodeIds)> {
-    let mut subtrees = HashMap::new();
+) -> DftHashMap<usize, (tree_sitter::Tree, TreeSitterConfig, HighlightedNodeIds)> {
+    let mut subtrees = DftHashMap::default();
 
     for language in &config.sub_languages {
         let mut query_cursor = tree_sitter::QueryCursor::new();
@@ -1374,7 +1379,7 @@ fn all_syntaxes_from_cursor<'a>(
     error_count: &mut usize,
     config: &TreeSitterConfig,
     highlights: &HighlightedNodeIds,
-    subtrees: &HashMap<usize, (tree_sitter::Tree, TreeSitterConfig, HighlightedNodeIds)>,
+    subtrees: &DftHashMap<usize, (tree_sitter::Tree, TreeSitterConfig, HighlightedNodeIds)>,
     ignore_comments: bool,
 ) -> Vec<&'a Syntax<'a>> {
     let mut result: Vec<&Syntax> = vec![];
@@ -1410,7 +1415,7 @@ fn syntax_from_cursor<'a>(
     error_count: &mut usize,
     config: &TreeSitterConfig,
     highlights: &HighlightedNodeIds,
-    subtrees: &HashMap<usize, (tree_sitter::Tree, TreeSitterConfig, HighlightedNodeIds)>,
+    subtrees: &DftHashMap<usize, (tree_sitter::Tree, TreeSitterConfig, HighlightedNodeIds)>,
     ignore_comments: bool,
 ) -> Option<&'a Syntax<'a>> {
     let node = cursor.node();
@@ -1426,7 +1431,7 @@ fn syntax_from_cursor<'a>(
             error_count,
             subconfig,
             subhighlights,
-            &HashMap::new(),
+            &DftHashMap::default(),
             ignore_comments,
         );
     }
@@ -1467,7 +1472,7 @@ fn list_from_cursor<'a>(
     error_count: &mut usize,
     config: &TreeSitterConfig,
     highlights: &HighlightedNodeIds,
-    subtrees: &HashMap<usize, (tree_sitter::Tree, TreeSitterConfig, HighlightedNodeIds)>,
+    subtrees: &DftHashMap<usize, (tree_sitter::Tree, TreeSitterConfig, HighlightedNodeIds)>,
     ignore_comments: bool,
 ) -> &'a Syntax<'a> {
     let root_node = cursor.node();
